clear
clc
close all

addpath ../util

% Setup
nData = 100;
amplitude = 1;
nMc = 1000;
snrList = (-10:1:10)';
noiseVariances = amplitude^2*10.^(-snrList/10);
nSnrs = length(snrList);

% compute the CRLB for the freqeuncy (in radians/sample)
crlb = 6*noiseVariances/(amplitude^2*nData*(nData^2-1));

% accuracy of the grid+refinement
refSnr = 30; % dB
accuracy = sqrt(10^(-refSnr/10)*6/(nData*(nData^2-1))); % CRLB @ 40 dB
dbGainList = [10, 3, 1, 0.1, 0.01, 0.001];
gainList = 10.^(dbGainList/10);
gridSizeList = [2*sqrt(12*(gainList-1)./gainList)/nData,...
    accuracy];
nGridSizes = length(gridSizeList);

% run the Monte Carlo Simulation
estimationError = nan(nMc, nSnrs, nGridSizes);
computationTime = nan(nMc, nSnrs, nGridSizes);
nGridPoints = nan(1, nGridSizes);
for iSnr = 1:nSnrs
    snrList(iSnr)
    for jMc = 1:nMc
        % generate the signal
        trueFreq = rand(1)*2*pi;
        phase = rand(1)*2*pi;
        complexSinusoid = exp(1i*trueFreq*(0:nData-1)');
        complexWeight = amplitude*exp(1i*phase);
        noiseFreeSignal = complexSinusoid*complexWeight;
        noise = sqrt(noiseVariances(iSnr)/2)*randn(nData,2)*[1;1i];
        data = noiseFreeSignal+noise;
        for kGridSize = 1:nGridSizes
            nGridPoints(kGridSize) = ceil(2*pi/gridSizeList(kGridSize));
            freqGrid = 2*pi*(0:(nGridPoints(kGridSize)-1))'/...
                nGridPoints(kGridSize);
            % compute the estimate on a grid
            tic
            gridCostFunction = abs(fft(data,nGridPoints(kGridSize))).^2;
            [~,idx] = max(gridCostFunction);
            gridEstimate = freqGrid(idx(1));
            % refine the estimate (only refine if the grid is too coarse)
            freqSpacing = diff(freqGrid(1:2));
            if accuracy < freqSpacing
                initialLowerBound = gridEstimate - freqSpacing;
                initialUpperBound = gridEstimate + freqSpacing;
                objFunc = @(freq) -periodogramCostFunction(freq, data);
                [finalLower, finalUpper] = fibonacciSearch(objFunc, ...
                    initialLowerBound, initialUpperBound, accuracy);
                estimatedFreq = (finalUpper+finalLower)/2;
            else
                estimatedFreq = gridEstimate;
            end
            computationTime(jMc,iSnr,kGridSize) = toc;
            % to avoid large errors due to the wrap around effect of the
            % frequency, we find the minimum error to three true freqs
            candidateErrors = estimatedFreq-trueFreq-[-2*pi;0;2*pi];
            [~,idx] = min(candidateErrors.^2);
            estimationError(jMc,iSnr,kGridSize) = candidateErrors(idx(1));
        end
    end
end

%% Compute the average error and computation time
rmse = nan(nSnrs, nGridSizes);
minComputationTime = nan(nSnrs, nGridSizes);
for kGridSize = 1:nGridSizes
    rmse(:,kGridSize) = sqrt(mean(estimationError(:,:,kGridSize).^2))';
    minComputationTime(:,kGridSize) = min(computationTime(:,:,kGridSize))';
end
avgComputationTime = mean(minComputationTime);
%% plot the results

figure(1)
semilogy(snrList,[sqrt(crlb),rmse])
xlabel('snr [dB]')
ylabel('RMSE [cycles/sample]')
legend(['CRLB';cellstr(num2str(gainList', 'g = %-d'));'No refinement'])

%% store the data
comment = ['The RMSE vs SNR for various grid sizes. The gains are ', num2str(gainList, 'g = %-d ')];
rmseData = [snrList, sqrt(crlb), rmse];
labels = 'snr crlb gridA gridB gridC gridD gridE gridF fftOnly';
mtxToTblPgfplots('periodogram',comment,labels,rmseData)
