function [costFunctionMatrix,fullPitchGrid] = ...
        singlePitchMlComplexCostFunction(data,maxModelOrder, ...
                                         resolution,pitchBounds, varargin)
% A fast implementation of the single pitch ML cost-function.
% The cost function is computed on a uniform (Fourier) grid for all model
% orders up to the model orders specified by maxModelOrder.
%
% * data: A data vector of complex-valued data.
% * maxModelOrder: The maximum number of harmonic components that the cost
%   function is evaluated for.
% * resolution: The resolution of the uniform pitch grid that the cost functions
%   are evaluated for.
% * pitchBounds: A 2D vector specifiyng the lower and upper bound for the pitch
%   in cycles/sample.
% * costFunctionMatrix: A matrix of cost functions. Each row corresponds to the 
%   the cost function for a candidate model order.
% * fullPitchGrid: The pitches corresponding to the values in the rows of the 
%   cost function matrix.
%
% if length(varargin) > 0 then it uses a C++ implementation.
%   in this case the costFunctionMatrix is transposed on output
%
%    Fast fundamental frequency estimation 
%    Copyright (C) 2015 Jesper K Nielsen and Tobias L. Jensen
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Contact: 
%      Jesper K Nielse, jkn@es.aau.dk
%      Tobias L. Jensen, tlj@es.aau.dk
%
    
    nData = length(data);

    % initialisation
    nFftGrid = ceil(1/resolution);
    dftData = fft(data,nFftGrid,1);

    minFftIndex = ceil(nFftGrid*pitchBounds(1));
    maxFftIndex = floor(nFftGrid*pitchBounds(2));
    validFftIndices = (minFftIndex:maxFftIndex)';
    fullPitchGrid = validFftIndices/nFftGrid;
    pitchGrid = fullPitchGrid;

    if length(varargin) == 1
        costFunctionMatrix = ...
        singlePitchMlComplexCostFunctionCustom(dftData, maxModelOrder,...
                                               nFftGrid, nData, ...
                                               pitchBounds);
    else
    
    	nPitches = length(pitchGrid);
    	crossCorrelationMatrix = nan(maxModelOrder,nPitches);
    	costFunctionMatrix = nan(maxModelOrder,nPitches);
    	% compute the FFT used in the approximate cost function
    	dftData = fft(data,nFftGrid,1);
    	dftDataMatrix = nan(maxModelOrder,nPitches);
        fftShiftVector = ...
            exp(1i*2*pi*validFftIndices*(nData-1)/(2*nFftGrid));
    	dftDataMatrix(1,:) = (fftShiftVector.*dftData(validFftIndices+1)).';
    	% Manually compute the value of the cost function for a modelorder of 1
    	crossCorrelationMatrix(1,:) = ...
    	    sin(pi*nData*pitchGrid')./sin(pi*pitchGrid');
    	gamma = ones(1,nPitches)/nData;
    	alpha = -gamma.*crossCorrelationMatrix(1,:);
    	durbinSolution = alpha;
    	levinsonSolution = gamma.*dftDataMatrix(1,:);
    	costFunctionMatrix(1,:) = gamma.*abs(dftDataMatrix(1,:)).^2;
    	% Recursively compute the remaining elements using the Levinson
    	% algorithm
    	for iOrder=2:maxModelOrder
    	    % precompute some quantities
    	    maxFftIndex = floor(min(nFftGrid*pitchBounds(2),nFftGrid/iOrder-1));
    	    nPitches = nPitches-(validFftIndices(end)-maxFftIndex);
    	    validPitchIndices = 1:nPitches;
    	    validFftIndices = validFftIndices(validPitchIndices);
    	    pitchGrid = pitchGrid(validPitchIndices);
    	    crossCorrelationMatrix(iOrder,validPitchIndices) = ...
    	        sin(pi*nData*iOrder*pitchGrid')./sin(pi*iOrder*pitchGrid');
    	   fftShiftVector = ...
    	        exp(1i*2*pi*validFftIndices*iOrder*(nData-1)/(2*nFftGrid));
    	    dftDataMatrix(iOrder,validPitchIndices) = ...
    	        (fftShiftVector.*dftData(validFftIndices*iOrder+ ...
                                         1)).';
    	    % run the first part of an iteration of the Levinson algorithm
    	    gamma = gamma(validPitchIndices)./(1-alpha(validPitchIndices).^2);
    	    mu = gamma.*...
    	            (dftDataMatrix(iOrder,validPitchIndices)-...
    	            sum(crossCorrelationMatrix(1:iOrder-1,validPitchIndices).*...
    	            levinsonSolution(iOrder-1:-1:1,validPitchIndices),1));
    	    w = durbinSolution(iOrder-1:-1:1,validPitchIndices);
    	    levinsonSolution = [levinsonSolution(:,validPitchIndices)+...
    	        (ones(iOrder-1,1)*mu).*w;mu];
    	    % compute the cost function
    	    costFunctionMatrix(iOrder,validPitchIndices) = ...
    	        real(sum(conj(dftDataMatrix(1:iOrder,validPitchIndices)).*...
    	        levinsonSolution(:,validPitchIndices),1));
    	    % run the Durbin part of an iteration of the Levinson
            % algorithm
    	    if iOrder < maxModelOrder
    	        alpha = -gamma.*...
    	            (crossCorrelationMatrix(iOrder,validPitchIndices)+...
    	            sum(crossCorrelationMatrix(1:iOrder-1,...
    	            validPitchIndices).*w,1));
    	        durbinSolution = [durbinSolution(:,validPitchIndices)+...
    	            (ones(iOrder-1,1)*alpha).*w;alpha];
    	    end
    	end
    end
end
